# Copyright 2011 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from webob import exc

from nova import exception
from nova import flags
from nova import log as logging
from nova import wsgi
from nova.api import keys
from nova.api.openstack import common
from nova.api.openstack import faults


LOG = logging.getLogger("nova.api.keys")

FLAGS = flags.FLAGS


def _translate_detail_keys(inst):
    """ Maps keys for details view"""

    inst_dict = _translate_summary_keys(inst)

    # No additional data / lookups at the moment

    return inst_dict


def _translate_summary_keys(inst):
    """ Maps keys for summary view"""
    inst_dict = {}
    inst_dict['id'] = inst['id']
    inst_dict['name'] = inst['name']
    inst_dict['fingerprint'] = inst['fingerprint']

    return inst_dict


class Controller(wsgi.Controller):
    """ The (crypto) Key API controller for the OpenStack API """

    _serialization_metadata = {
        'application/xml': {
            "attributes": {
                "key": ["id", "keyName", "keyFingerprint", "privateKey"]}}}

    def __init__(self):
        self.keys_api = keys.API()
        super(Controller, self).__init__()

    def show(self, req, id):
        """Return data about the given key"""
        context = req.environ['nova.context']

        try:
            key = self.keys_api.get_key_by_id(context, id)
        except exception.NotFound:
            return faults.Fault(exc.HTTPNotFound())

        return {'key': _translate_detail_keys(key)}

    def delete(self, req, id):
        """ Delete a key """
        context = req.environ['nova.context']
        try:
            key = self.keys_api.get_key_by_id(context, id)
            self.keys_api.delete_key_pair(context, key)
        except exception.NotFound:
            return faults.Fault(exc.HTTPNotFound())
        return exc.HTTPAccepted()

    def index(self, req):
        """ Returns a summary list of keys for a given user """
        return self._items(req, entity_maker=_translate_summary_keys)

    def detail(self, req):
        """ Returns a detailed list of keys for a given user """
        return self._items(req, entity_maker=_translate_detail_keys)

    def _items(self, req, entity_maker):
        """Returns a list of keys for a given user.

        entity_maker - applied to each item to map to return format
        """
        context = req.environ['nova.context']

        key_pairs = self.keys_api.get_all_keys(context)
        limited_list = common.limited(key_pairs, req)
        res = [entity_maker(inst) for inst in limited_list]
        return {'keys': res}

    def create(self, req):
        """ Creates a new server for a given user """
        context = req.environ['nova.context']

        env = self._deserialize(req.body, req)
        if not env:
            return faults.Fault(exc.HTTPUnprocessableEntity())

        key_name = env['key']['name']

        data = self.keys_api.create_key_pair(context, key_name)
        private_key = data['private_key']
        key = data['key']

        retval = _translate_detail_keys(key)
        retval['privateKey'] = private_key

        return {'key': retval}
