# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 Justin Santa Barbara
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Provides common functionality for integrated unit tests
"""

import random
import string

from nova import apiservice
from nova import exception
from nova import flags
from nova import service
from nova.api import openstack  # For FLAGS.allow_testing_api
from nova.auth import manager
from nova.exception import Error
from nova.log import logging
from nova.tests.integrated.api import client

FLAGS = flags.FLAGS


def generate_random_alphanumeric(length):
    """Creates a random string of specified length"""
    return ''.join(random.choice(string.ascii_uppercase + string.digits)
                   for _x in range(length))


def generate_new_element(items, prefix):
    """Creates a random string with prefix, that is not in 'items' list"""
    while True:
        candidate = prefix + generate_random_alphanumeric(8)
        if not candidate in items:
            return candidate
        print "Random collision on %s" % candidate


class IntegratedUnitTestContext(object):
    __INSTANCE = None

    def __init__(self):
        self.auth_manager = manager.AuthManager()

        self.wsgi_server = None
        self.wsgi_apps = []
        self.api_service = None

        self.saved_flags = {}
        self.services = []
        self.auth_url = None
        self.auth_user = None
        self.auth_key = None
        self.project_name = None
        self.openstack_api = None

        self.setup()

    def setup(self):
        self._configure_flags()

        self._create_test_user()

        self._start_services()

    def _configure_flags(self):
        self._overwrite_flag('allow_testing_api', True)
        if not FLAGS.allow_testing_api:
            # Sanity check!
            raise exception.Error("Flag override failed")

        # NOTE(justinsb): I don't think the services are shutting down their
        # fake_rabbit connections properly, so fake_rabbit=True breaks.
        # Works with the real thing though
        self._overwrite_flag('fake_rabbit', False)
        self._overwrite_flag('volume_driver',
                             'nova.volume.driver.LoggingVolumeDriver')

    def _overwrite_flag(self, flag_key, flag_value):
        self.saved_flags[flag_key] = FLAGS[flag_key].value
        FLAGS[flag_key].value = flag_value

    def _restore_flags(self):
        if self.saved_flags:
            for k, v in self.saved_flags.iteritems():
                FLAGS[k].value = v
        self.saved_flags.clear()

    def _create_test_user(self):
        self.auth_user, self.auth_key = self._create_unittest_user()

        # No way to currently pass this through the OpenStack API
        self.project_name = 'openstack'
        self._configure_project(self.project_name)

    def _start_services(self):
        # WSGI shutdown broken :-(
        if not self.api_service:
            self._start_api_service()

        self._start_volume_service()
        self._start_scheduler_service()

        self.openstack_api = client.TestOpenStackClient(self.auth_user,
                                                        self.auth_key,
                                                        self.auth_url)

    def cleanup(self):
        for service in self.services:
            service.kill()
        self.services = []
        # TODO(justinsb): Shutdown WSGI & anything else we startup
        # WSGI shutdown broken :-(
        # self.wsgi_server.terminate()
        # self.wsgi_server = None
        self._restore_flags()
        self.openstack_api = None

    def _create_unittest_user(self):
        users = self.auth_manager.get_users()
        user_names = [user.name for user in users]
        auth_name = generate_new_element(user_names, 'unittest_user_')
        auth_key = generate_random_alphanumeric(16)

        # Right now there's a bug where auth_name and auth_key are reversed
        auth_key = auth_name

        self.auth_manager.create_user(auth_name, auth_name, auth_key, False)
        return auth_name, auth_key

    def _configure_project(self, project_name):
        projects = self.auth_manager.get_projects()
        project_names = [project.name for project in projects]
        if not project_name in project_names:
            project = self.auth_manager.create_project(project_name,
                                                       self.auth_user,
                                                       description=None,
                                                       member_users=None)
        else:
            self.auth_manager.add_to_project(self.auth_user, project_name)

    def _start_api_service(self):
        api_service = apiservice.ApiService.create()
        api_service.start()

        if not api_service:
            raise Exception("API Service was None")

        # WSGI shutdown broken :-(
        #self.services.append(volume_service)
        self.api_service = api_service

        self.auth_url = 'http://localhost:8774/v1.0'

        return api_service

    def _start_volume_service(self):
        volume_service = service.Service.create(binary='nova-volume')
        volume_service.start()
        self.services.append(volume_service)
        return volume_service

    def _start_scheduler_service(self):
        scheduler_service = service.Service.create(binary='nova-scheduler')
        scheduler_service.start()
        self.services.append(scheduler_service)
        return scheduler_service

    def get_openstack_api(self):
        return self.openstack_api

    # WSGI shutdown broken :-(
    #@staticmethod
    #def get():
    #    if not IntegratedUnitTestContext.__INSTANCE:
    #        IntegratedUnitTestContext.startup()
    #        #raise Error("Must call IntegratedUnitTestContext::startup")
    #    return IntegratedUnitTestContext.__INSTANCE

    @staticmethod
    def startup():
        # Because WSGI shutdown is broken at the moment, we have to recycle
        if IntegratedUnitTestContext.__INSTANCE:
            #raise Error("Multiple calls to IntegratedUnitTestContext.startup")
            IntegratedUnitTestContext.__INSTANCE.setup()
        else:
            IntegratedUnitTestContext.__INSTANCE = IntegratedUnitTestContext()
        return IntegratedUnitTestContext.__INSTANCE

    @staticmethod
    def shutdown():
        if not IntegratedUnitTestContext.__INSTANCE:
            raise Error("Must call IntegratedUnitTestContext::startup")
        IntegratedUnitTestContext.__INSTANCE.cleanup()
        # WSGI shutdown broken :-(
        #IntegratedUnitTestContext.__INSTANCE = None
